/******************************************************************************/
// dwfbfile.h - library for r/w of DK2 DWFB/WAD files.
/******************************************************************************/
// Author:   Tomasz Lis
// Created:  15 Jul 2008

// Purpose:
//   Header file. Defines exported routines from dwfbfile.c

// Comment:
//   None.

//Copying and copyrights:
//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation; either version 2 of the License, or
//   (at your option) any later version.
/******************************************************************************/

#ifndef DWFBFILE_H
#define DWFBFILE_H

# include <stdio.h>

#define DWFBFLAG_VERBOSE        0x01
#define DWFBFLAG_FORCE_LOADNAME 0x02
#define DWFBFLAG_FORCE_LOADDATA 0x04
#define DWFBFLAG_IGNORE_BADHDR  0x40

#define DWFB_COMPRESS_NONE      0x00
#define DWFB_COMPRESS_MTZ4      0x04

#pragma pack(1)

struct DWFB_Header {
    char magic[4];              // Magic value (DWFB)
    unsigned int version;       // Version (2)
    unsigned char padding[64];  // zeros
    unsigned int dirnum;        // Number Of Files
    unsigned int name_offs;     // Null terminated filenames Directory Offset
    unsigned int name_len;      // Filenames Directory Length
    unsigned int unknown;
    };
#define SIZEOF_DWFB_Header 88

struct DWFB_Entry {
    unsigned int unknown;       // unknown nonzero value !?
    unsigned int name_offs;     // Offset of null-terminated name string
    unsigned int name_len;      // Name string length
    unsigned int body_offs;     // Offset to the file body
    unsigned int body_len;      // Compressed size in bytes
    unsigned int store_method;  // Compression method: 0 = None, 4 = Unknown compression method
    unsigned int original_len;  // Decompressed size in bytes
    unsigned char padding[12];  // zeros
    // standard DWFB entry ends here
    char *name;
    unsigned char *body;        // Pointer to loaded DWFB data
    };
#define SIZEOF_DWFB_Entry 40

struct DWFB_File {
    struct DWFB_Header header;
    struct DWFB_Entry **entries;
    FILE *fp;
    unsigned long iosize;
    long disksize;
    };

#pragma pack()

// Routines

short dwfb_error(const char *msg);
short dwfb_ferror(const char *format, ...);
short dwfb_print_header(const struct DWFB_Header *hdr);
short dwfb_print_entry(const struct DWFB_Entry *entry);

long dwfb_fread_entry_body(struct DWFB_File *dwfbfile,struct DWFB_Entry *entry,short flags);
long dwfb_fread_entry_name(struct DWFB_File *dwfbfile,struct DWFB_Entry *entry,short flags);

char *dwfb_entry_name(struct DWFB_File *dwfbfile,struct DWFB_Entry *entry,short flags);
char *dwfb_entryidx_name(struct DWFB_File *dwfbfile,unsigned long idx,short flags);
char *dwfb_entry_body(struct DWFB_File *dwfbfile,struct DWFB_Entry *entry,short flags);
char *dwfb_entryidx_body(struct DWFB_File *dwfbfile,unsigned long idx,short flags);

struct DWFB_File *dwfb_open(char *fname,short flags);
short dwfb_close(struct DWFB_File *dwfbfile,short flags);


#endif
