/******************************************************************************/
// dwfbtool.c - Dungeon Keeper 2 Tools.
/******************************************************************************/
// Author:   Tomasz Lis
// Created:  10 Jun 2008

// Purpose:
//   Allows reading/writing DK2 DWFB files (WADs).

// Comment:
//   None.

//Copying and copyrights:
//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation; either version 2 of the License, or
//   (at your option) any later version.
/******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "dwfbfile.h"

int main(int argc, char *argv[])
{
    printf("\nDungeon Keeper 2 DWFB/WAD tool\n");
    printf("-------------------------------\n");
    if ((argc<3)||(strlen(argv[2])!=1))
    {
        printf("Not enought parameters.\n");
        printf("Usage:\n");
        printf("  %s <wadfile> <operation>\n","dwfbtool");
//        printf("  %s <wadfile> <operation> [files]\n","dwfbtool");
        printf("The <wadfile> should be given without extension.\n");
        printf("Valid <operations> are:\n");
        printf("  x: eXtract files to folder of same name\n");
        printf("  c: Create the wad archive using folder\n");
        printf("  d: Dump file structure data\n");
//        printf("  a: Add files to archive                l: List contents of archive\n");
//        printf("  d: Delete files from archive           t: Test integrity of archive\n");
//        printf("  e: Extract files from archive          x: eXtract files with full pathname\n");
        printf("\n");
        system("PAUSE");	
    	return 1;
    }
  struct DWFB_File *dwfbfile;
  short flags = DWFBFLAG_VERBOSE|DWFBFLAG_FORCE_LOADNAME|DWFBFLAG_FORCE_LOADDATA;
  char *dwfbfname=argv[1];
  char operatn=tolower(argv[2][0]);
  dwfbfile=dwfb_open(dwfbfname,flags);
  if (dwfbfile==NULL)
  {
    system("PAUSE");	
    return 2;
  }
  switch (operatn)
  {
  case 'd':
    {
      dwfb_print_header(&(dwfbfile->header));
      int i;
      for (i=0;i<dwfbfile->header.dirnum;i++)
      {
        printf(" Entry %d:\n",i);
        dwfb_print_entry(dwfbfile->entries[i]);
      }
      printf(" Processed %lu out of %ld file bytes.\n",dwfbfile->iosize,dwfbfile->disksize);
    };break;
  case 'a':
      dwfb_error("Adding files not supported yet");
      break;
  case 't':
      dwfb_error("Testing not supported yet");
      break;
  case 'l':
      dwfb_error("Listing not supported yet");
      break;
  case 'c':
      dwfb_error("Creating WADs not supported yet");
      break;
  case 'e':
  case 'x':
      dwfb_error("Extracting not supported yet");
      break;
  default:
      printf("Unknown opertation symbol.\n");
      printf("Exiting without any changes.\n");
      break;
  }
  if (!dwfb_close(dwfbfile,flags))
    return 3;
  return 0;
}
